struct stringlist {
	struct stringlist *next;
	char *s;
};

struct path_info {
	struct path_info *next;
	char *name;
	dev_t device;
	ino_t inode;
	int link;
};

int checksum(char *block, int sgn);
int checksum_is_valid(char *buf, char *filename);

/* -1: error, 0: success, 1: success for directory */
int prepare_single_path_info(struct path_info **pi, char *name, int linkfirst);

int prepare_path(struct path_info **pi, char *parent, char *path,
	int linkfirst);

/* 0: success, -1: failure */
int output_path(FILE *out, char *outfn, struct path_info *pi,
	struct path_info *opi, struct stringlist *untracked, uid_t uid,
	char *uname, gid_t gid, char *gname);

void output_footer(FILE *out, char *filename);

#define EXTRACT_KEEP 1
#define EXTRACT_PERMISSION 2
#define EXTRACT_NOISYKEEP 4

/* -1: error, 0: containing folder missing, 1: nothing done, 2: success */
int ex_write(FILE *in, char *infn, char *filename, mode_t mode, time_t modtime,
	size_t size, int flags, int reopen, char *tgtdir);
/* -1: error, 0: containing folder missing, 1: nothing done, 2: success */
int ex_symlink(char *target, char *filename, int flags, char *tgtdir);
/* -1: error, 0: containing folder missing, 1: nothing done, 2: success */
int ex_link(char *target, char *filename, int flags, char *tgtdir);
/* -1: error, 0: containing folder missing, 1: nothing done, 2: success */
int ex_mknod(char *filename, mode_t mode, dev_t dev, int flags, char *tgtdir);
/* -1: error, 0: containing folder missing, 1: nothing done, 2: success */
int ex_mkdir(char *filename, mode_t mode, int flags, char *tgtdir);

struct file_list {
	struct file_list *next;
	int dir;
	char *filename;
	char *backup;
};

/* 0: success, -1: failure */
int guarantee_parent_dir(char *tgtdir, struct file_list *oldfl,
	struct file_list **newfl, int flags, int force, int overwrite,
	int tracking, char *dpytgtdir, char *filename, mode_t mode, int track);

/* 0: success, -1: failure */
int extractor_write(char *tgtdir, struct file_list *oldfl,
	struct file_list **newfl, int flags, int force, int overwrite,
	int tracking, FILE *in, char *infn, char *dpytgtdir, char *filename,
	mode_t mode, time_t modtime, size_t size, int track, int reopen);

/* 0: success, -1: failure */
int extractor_link(char *tgtdir, struct file_list *oldfl,
	struct file_list **newfl, int flags, int force, int overwrite,
	int tracking, char *dpytgtdir, char *filename, char *target, int track);

/* 0: success, -1: failure */
int extractor_symlink(char *tgtdir, struct file_list *oldfl,
	struct file_list **newfl, int flags, int force, int overwrite,
	int tracking, char *dpytgtdir, char *filename, char *target, int track);

/* 0: success, -1: failure */
int extractor_mknod(char *tgtdir, struct file_list *oldfl,
	struct file_list **newfl, int flags, int force, int overwrite,
	int tracking, char *dpytgtdir, char *filename, dev_t dev, mode_t mode,
	int track);

/* 0: success, -1: failure */
int guarantee_dir(char *tgtdir, struct file_list *oldfl,
	struct file_list **newfl, int flags, int force, int overwrite,
	int tracking, char *dpytgtdir, char *filename, mode_t mode, int track);

/* 1: file processed successfully, 0: no more files,
	-1: file processed with errors */
int extract_file(char *tgtdir, struct file_list *oldfl,
	struct file_list **newfl, int flags, int force, int overwrite,
	int tracking, char *dpytgtdir, FILE *in, char *infn, int trackable);

/* 1: file processed successfully, 0: no more files,
	-1: file processed with errors */
int list_file(FILE *in, char *infn, int trackable);

struct file_list *file_list_add(struct file_list *fl, int dir, char *tgtdir,
	char *filename);
