#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>

#include "lib.h"
#include "archive.h"
#include "package.h"

void list_dep(char **dep, char **max, char **min) {
	if (*dep) {
		printf("Dependency: %s", *dep);
		free(*dep);
		*dep = NULL;
		if (*min) {
			printf(" minimum version %s", *min);
			free(*min);
			*min = NULL;
		}
		if (*max) {
			printf(" maximum version %s", *max);
			free(*max);
			*max = NULL;
		}
		printf("\n");
	}
}

int main(int argc, char **argv) {
	struct stringlist *sl, *csle;
	FILE *f;
	char *key, *value, *datafile, *name, *target, *dep, *min, *max;
	int i, iinc, j, files;
	progname = argv[0];
	name = NULL;
	target = "/";
	files = 0;
	for (i = 1; i < argc; i += iinc + 1) {
		iinc = 0;
		if (argv[i][0] == '-' && argv[i][1]) {
			for (j = 1; argv[i][j]; j++) {
				switch (argv[i][j]) {
				case 'f':
					files = 1;
					break;
				case 't':
					if (i + iinc + 1 >= argc || argv[i+iinc+1][0] == '-') {
						fprintf(stderr, "%s: -t requires argument\n",
							progname);
						exit(1);
					}
					iinc++;
					target = argv[i + iinc];
					break;
				case 'V':
					version("rmpak");
					return 0;
				case '?':
					version("rmpak");
					fprintf(stderr,
						"\n%s [-f] [-t TARGET] PACKAGE\n"
						"\tDescribe the installed package named PACKAGE. If\n"
						"\t-t is specified, treat TARGET as the root\n"
						"\tdirectory. If -f is specified, list installed\n"
						"\tfiles.\n"
						"%s -?\n"
						"\tShow this usage message.\n"
						"%s -V\n"
						"\tDisplay the version of this command.\n",
						progname, progname, progname);
					return 0;
				default:
					fprintf(stderr, "%s: invalid -%c\n", progname, argv[i][j]);
					exit(1);
				}
			}
		} else {
			if (name) {
				fprintf(stderr,
					"%s: invalid %s: too many package names on command line\n",
					progname, argv[i]);
				exit(1);
			}
			name = argv[i];
		}
	}
	if (!name) {
		fprintf(stderr, "%s: package name required\n", progname);
		exit(1);
	}
	if (chdir(target))
		die(target);
	f = open_package_datafile(name, target, &datafile);
	if (!f) {
		fprintf(stderr, "%s: package %s not installed\n", progname, name);
		return 1;
	}
	printf("Name: %s\n", name);
	sl = NULL;
	dep = NULL;
	while (read_kvp(f, datafile, &key, &value)) {
		if (files && (!strcmp(key, "File") || !strcmp(key, "Directory"))) {
			csle = xmalloc(sizeof(struct stringlist), datafile);
			csle->s = value;
			csle->next = sl;
			sl = csle;
		} else if (!strcmp(key, "Version")) {
			printf("Version: %s\n", value);
			free(value);
		} else if (!strcmp(key, "Dependency")) {
			list_dep(&dep, &min, &max);
			dep = value;
			min = NULL;
			max = NULL;
		} else if (!strcmp(key, "Minimum")) {
			if (dep) {
				if (min)
					free(min);
				min = value;
			} else {
				free(value);
			}
		} else if (!strcmp(key, "Maximum")) {
			if (dep) {
				if (max)
					free(max);
				max = value;
			} else {
				free(value);
			}
		}
		free(key);
	}
	list_dep(&dep, &min, &max);
	cleanup_package_datafile(f, target, datafile);
	if (files) {
		printf("\nFiles:\n");
		for (csle = sl; csle; csle = csle->next)
			printf("%s\n", csle->s);
	}
	return 0;
}
