#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <unistd.h>
#include <fcntl.h>
#include <dirent.h>
#include <sys/types.h>

#include "lib.h"
#include "archive.h"
#include "package.h"

int main(int argc, char **argv) {
	struct dirent *de;
	DIR *d;
	FILE *f;
	char *key, *value, *datafile, *target;
	int i, iinc, j, found;
	progname = argv[0];
	target = "/";
	for (i = 1; i < argc; i += iinc + 1) {
		iinc = 0;
		if (argv[i][0] == '-' && argv[i][1]) {
			for (j = 1; argv[i][j]; j++) {
				switch (argv[i][j]) {
				case 't':
					if (i + iinc + 1 >= argc || argv[i+iinc+1][0] == '-') {
						fprintf(stderr, "%s: -t requires argument\n",
							progname);
						exit(1);
					}
					iinc++;
					target = argv[i + iinc];
					break;
				case 'V':
					version("lspak");
					return 0;
				case '?':
					version("lspak");
					fprintf(stderr,
						"\n%s [-t TARGET]\n"
						"\tList installed packages. If -t is\n"
						"\tspecified, treat TARGET as the root directory.\n"
						"%s -?\n"
						"\tShow this usage message.\n"
						"%s -V\n"
						"\tDisplay the version of this command.\n",
						progname, progname, progname);
					return 0;
				default:
					fprintf(stderr, "%s: invalid -%c\n", progname, argv[i][j]);
					exit(1);
				}
			}
		} else {
			fprintf(stderr, "%s: invalid %s\n", progname, argv[i]);
			exit(1);
		}
	}
	if (chdir(target))
		die(target);
	d = opendir("etc/tspm");
	if (!d)
		target_die(target, "etc/tspm");
	for (de = readdir(d); de; de = readdir(d)) {
		if (strcmp(de->d_name, ".") && strcmp(de->d_name, "..")) {
			f = open_package_datafile(de->d_name, target, &datafile);
			if (f) {
				found = 0;
				while (read_kvp(f, datafile, &key, &value)) {
					if (!strcmp(key, "Version")) {
						printf("%s version %s\n", de->d_name, value);
						found = 1;
					}
					free(key);
					free(value);
					if (found)
						break;
				}
				cleanup_package_datafile(f, target, datafile);
			}
			if (!found) {
				fprintf(stderr, "%s: internal record store corrupted: "
					"no version number for %s\n", progname, de->d_name);
				printf("%s\n", de->d_name);
			}
		}
	}
	return 0;
}
