#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <unistd.h>
#include <fcntl.h>
#include <dirent.h>
#include <sys/types.h>

#include "lib.h"
#include "fs.h"
#include "archive.h"
#include "package.h"

void dependency_check(char *name, char *depname, char *target) {
	char *datafile, *key, *value;
	FILE *f;
	if (strcmp(name, depname)) {
		f = open_package_datafile(depname, target, &datafile);
		if (f) {
			while (read_kvp(f, datafile, &key, &value)) {
				if (!strcmp(key, "Dependency") && !strcmp(value, name)) {
					fprintf(stderr, "%s: package %s depends on package %s\n",
						progname, depname, name);
					exit(1);
				}
				free(key);
				free(value);
			}
			cleanup_package_datafile(f, target, datafile);
		}
	}
}

int main(int argc, char **argv) {
	struct dirent *de;
	DIR *d;
	FILE *f;
	char *key, *value, *datafile, *name, *target;
	int i, iinc, j;
	progname = argv[0];
	name = NULL;
	target = "/";
	for (i = 1; i < argc; i += iinc + 1) {
		iinc = 0;
		if (argv[i][0] == '-' && argv[i][1]) {
			for (j = 1; argv[i][j]; j++) {
				switch (argv[i][j]) {
				case 't':
					if (i + iinc + 1 >= argc || argv[i+iinc+1][0] == '-') {
						fprintf(stderr, "%s: -t requires argument\n",
							progname);
						exit(1);
					}
					iinc++;
					target = argv[i + iinc];
					break;
				case 'V':
					version("rmpak");
					return 0;
				case '?':
					version("rmpak");
					fprintf(stderr,
						"\n%s [-t TARGET] PACKAGE\n"
						"\tRemove the package named PACKAGE. If -t is\n"
						"\tspecified, treat TARGET as the root directory.\n"
						"%s -?\n"
						"\tShow this usage message.\n"
						"%s -V\n"
						"\tDisplay the version of this command.\n",
						progname, progname, progname);
					return 0;
				default:
					fprintf(stderr, "%s: invalid -%c\n", progname, argv[i][j]);
					exit(1);
				}
			}
		} else {
			if (name) {
				fprintf(stderr,
					"%s: invalid %s: too many package names on command line\n",
					progname, argv[i]);
				exit(1);
			}
			name = argv[i];
		}
	}
	if (!name) {
		fprintf(stderr, "%s: package name required\n", progname);
		exit(1);
	}
	if (chdir(target))
		die(target);
	datafile = malloc(10 + strlen(argv[1]));
	if (!datafile) {
		fprintf(stderr, "%s: out of memory\n", progname);
		return 1;
	}
	sprintf(datafile, "etc/tspm/%s", name);
	errno = 0;
	f = fopen(datafile, "r");
	if (!f) {
		if (errno == ENOENT) {
			fprintf(stderr, "%s: package %s not installed\n", progname, name);
			return 1;
		} else {
			target_die(target, datafile);
		}
	}
	d = opendir("etc/tspm");
	if (!d)
		target_die(target, "etc/tspm");
	errno = 0;
	for (de = readdir(d); de; de = readdir(d)) {
		if (strcmp(de->d_name, ".") && strcmp(de->d_name, "..") &&
				strcmp(de->d_name, name)) {
			dependency_check(name, de->d_name, target);
			errno = 0;
		}
	}
	if (errno)
		target_die(target, "etc/tspm");
	closedir(d);
	while (read_kvp(f, datafile, &key, &value)) {
		if ((!strcmp(key, "File") || !strcmp(key, "Directory")) &&
				strcmp(value, ".") && delete(value, 0) && errno != ENOTEMPTY)
			target_error(target, value);
		free(key);
		free(value);
	}
	if (fclose(f))
		target_error(target, datafile);
	errno = 0;
	if (delete(datafile, 0))
		target_error(target, datafile);
	return 0;
}
