#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <fcntl.h>
#include <pwd.h>
#include <grp.h>
#include <sys/types.h>

#include "lib.h"
#include "archive.h"
#include "package.h"

int main(int argc, char **argv) {
	struct dep *deps, *cd;
	FILE *f;
	char *name, *vers, *datafile, *filename;
	int r, i, j, iinc, result;
	progname = argv[0];
	filename = NULL;
	result = 0;
	for (i = 1; i < argc; i += iinc + 1) {
		iinc = 0;
		if (argv[i][0] == '-' && argv[i][1]) {
			for (j = 1; argv[i][j]; j++) {
				switch (argv[i][j]) {
				case 'V':
					version("testpak");
					return 0;
				case '?':
					version("testpak");
					fprintf(stderr,
						"\n%s FILENAME\n"
						"\tList the contents of the package file named\n"
						"\tFILENAME, that is, test what would happen if the\n"
						"\tpackage were installed.\n"
						"%s -?\n"
						"\tShow this usage message.\n"
						"%s -V\n"
						"\tDisplay the version of this command.\n",
						progname, progname, progname);
					return 0;
				default:
					fprintf(stderr, "%s: invalid -%c\n", progname, argv[i][j]);
					exit(1);
				}
			}
		} else {
			if (filename) {
				fprintf(stderr,
					"%s: invalid %s: too many filenames on command line\n",
					progname, argv[i]);
				exit(1);
			}
			filename = argv[i];
		}
	}
	if (!filename) {
		fprintf(stderr, "%s: filename required\n", progname);
		exit(1);
	}
	f = xz_stream_open(filename, "r");
	if (!f)
		exit(1);
	deps = NULL;
	read_package_header(f, filename, &name, &vers, &deps);
	printf("Name: %s\n", name);
	printf("Version: %s\n", vers);
	for (cd = deps; cd; cd = cd->next) {
		printf("Dependency: %s", cd->name);
		if (cd->min_version)
			printf(" minimum version %s", cd->min_version);
		if (cd->max_version)
			printf(" maximum version %s", cd->max_version);
		printf("\n");
	}
	printf("\nFiles:\n");
	for (r = list_file(f, filename, 1); r; r = list_file(f, filename, 1))
		if (r < 0)
			result = 1;
	if (fclose(f)) {
		error(filename);
		result = 1;
	}
	return result;
}
